/*
 * agenerate.cpp
 *
 *  Created on: 16. 2. 2015
 *	  Author: Jan Travnicek
 */

#include <version.hpp>

#include <tclap/CmdLine.h>
#include <global/GlobalData.h>
#include <alib/measure>

#include <exception/CommonException.h>
#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>

int main ( int argc, char * argv[] ) {
	try {
		common::GlobalData::argc = argc;
		common::GlobalData::argv = argv;

		TCLAP::CmdLine cmd ( "String generate from grammar", ' ', ALIB_VERSION_INFO );

		std::vector < std::string > allowed;
		allowed.push_back ( "upTo" );
		allowed.push_back ( "CYK" );
		TCLAP::ValuesConstraint < std::string > allowedVals ( allowed );

		TCLAP::ValueArg < std::string > type ( "t", "type", "Run type", false, "accept", & allowedVals );
		cmd.add ( type );

		TCLAP::ValueArg < unsigned > upto ( "", "upto", "Generate all strings up to length", false, 5, "unsigned" );
		cmd.add ( upto );

		TCLAP::ValueArg < std::string > stringInput ( "s", "string", "String from file", false, "-", "file" );
		cmd.add ( stringInput );

		TCLAP::ValueArg < std::string > grammarInput ( "g", "grammar", "Grammar to use", false, "-", "file" );
		cmd.add ( grammarInput );

		TCLAP::SwitchArg measure ( "m", "measure", "Measure times", false );
		cmd.add ( measure );

		TCLAP::SwitchArg verbose ( "v", "verbose", "Be verbose", false );
		cmd.add ( verbose );

		cmd.parse ( argc, argv );

		if ( verbose.isSet ( ) )
			common::GlobalData::verbose = true;

		if ( measure.isSet ( ) )
			common::GlobalData::measure = true;

		cli::Environment environment;
		environment.setBinding ( "stringInput", stringInput.getValue ( ) );
		environment.setBinding ( "grammarInput", grammarInput.getValue ( ) );
		environment.setBinding ( "upTo", ext::to_string ( upto.getValue ( ) ) );
		environment.setBinding ( "stdout", "-" );

		measurements::start ( "Overal", measurements::Type::OVERALL );
		measurements::start ( "Input read", measurements::Type::AUXILIARY );
	
		cli::Parser parser ( cli::Lexer ( "execute <#grammarInput > $grammar" ) );
		parser.parse ( )->run ( environment );

		if ( type.getValue ( ) == "CYK" ) {
			parser = cli::Parser ( cli::Lexer ( "execute <#stringInput > $string" ) );
			parser.parse ( )->run ( environment );
		}

		if ( type.getValue ( ) == "upTo" ) {
			parser = cli::Parser ( cli::Lexer ( "execute (unsigned) #upTo > $upTo" ) );
			parser.parse ( )->run ( environment );
		}

		measurements::end ( );
		measurements::start ( "Algorithm", measurements::Type::MAIN );

		std::string cliCommand;
		if ( ( type.getValue ( ) == "upTo" ) ) {
			cliCommand = "execute grammar::generate::GenerateUpToLength $grammar $upTo > $output";
		} else if ( type.getValue ( ) == "CYK" ) {
			cliCommand = "execute grammar::generate::CockeYoungerKasami $grammar $string > $output";
		} else {
			throw exception::CommonException ( "Invalid run type" );
		}

		parser = cli::Parser ( cli::Lexer ( cliCommand ) );
		parser.parse ( )->run ( environment );

		measurements::end ( );
		measurements::start ( "Output write", measurements::Type::AUXILIARY );

		parser = cli::Parser ( cli::Lexer ( "execute $output >#stdout" ) );
		parser.parse ( )->run ( environment );

		measurements::end();
		measurements::end();

		if ( measure.getValue ( ) ) common::Streams::measure << measurements::results ( ) << std::endl;

		return 0;
	} catch ( const exception::CommonException & exception ) {
		factory::XmlDataFactory::toStdout ( exception );
		return 1;
	} catch ( const TCLAP::ArgException & exception ) {
		common::Streams::out << exception.error ( ) << std::endl;
		return 2;
	} catch ( const std::exception & exception ) {
		common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
		return 3;
	} catch ( ... ) {
		common::Streams::err << "Unknown exception caught." << std::endl;
		return 127;
	}
}
