/*
 * Registry.cpp
 *
 *  Created on: 10. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/Registry.h>

#include <registry/AlgorithmRegistry.hpp>
#include <registry/ImmediateRegistry.hpp>
#include <registry/ValuePrinterRegistry.hpp>
#include <registry/CastRegistry.hpp>
#include <registry/NormalizeRegistry.hpp>
#include <registry/ContainerRegistry.hpp>

namespace abstraction {

ext::set < ext::pair < std::string, ext::vector < std::string > > > Registry::listAlgorithmGroup ( const std::string & group ) {
	return AlgorithmRegistry::listGroup ( group );
}

ext::set < ext::pair < std::string, ext::vector < std::string > > > Registry::listAlgorithms ( ) {
	return AlgorithmRegistry::list ( );
}

ext::set < std::string > Registry::listCastsFrom ( const std::string & type ) {
	return CastRegistry::listFrom ( type );
}

ext::set < std::string > Registry::listCastsTo ( const std::string & type ) {
	return CastRegistry::listTo ( type );
}

ext::set < ext::pair < std::string, std::string > > Registry::listCasts ( ) {
	return CastRegistry::list ( );
}

ext::set < ext::tuple < AlgorithmCategories::AlgorithmCategory, ext::pair < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier > >, ext::vector < ext::tuple < std::string, ext::set < abstraction::ParamQualifiers::ParamQualifier >, std::string > > > > Registry::listOverloads ( const std::string & algorithm, const ext::vector < std::string > & templateParams ) {
	return AlgorithmRegistry::listOverloads ( algorithm, templateParams );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getContainerAbstraction ( const std::string & container, const std::string & type ) {
	return ContainerRegistry::getAbstraction ( container, type );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getAlgorithmAbstraction ( const std::string & name, const ext::vector < std::string > & templateParams, const ext::vector < std::string > & paramTypes, AlgorithmCategories::AlgorithmCategory category ) {
	return AlgorithmRegistry::getAbstraction ( name, templateParams, paramTypes, category );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getCastAbstraction ( const std::string & target, const std::string & param ) {
	return CastRegistry::getAbstraction ( target, param );
}

bool Registry::isCastNoOp ( const std::string & target, const std::string & param ) {
	return CastRegistry::isNoOp ( target, param );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getImmediateAbstraction ( const std::string & result, std::string value ) {
	return ImmediateRegistry::getAbstraction ( result, std::move ( value ) );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getNormalizeAbstraction ( const std::string & param ) {
	return NormalizeRegistry::getAbstraction ( param );
}

bool Registry::hasNormalize ( const std::string & param ) {
	return NormalizeRegistry::hasNormalize ( param );
}

std::shared_ptr < abstraction::OperationAbstraction > Registry::getValuePrinterAbstraction ( const std::string & param, std::ostream & os ) {
	return ValuePrinterRegistry::getAbstraction ( param, os );
}

} /* namespace abstraction */
