/*
 * GlushkovIndexate.cpp
 *
 *  Created on: 14. 4. 2016
 *	  Author: Tomas Pecka
 */

#include <alib/pair>

#include "GlushkovIndexate.h"

#include <rte/formal/FormalRTEElements.h>

namespace rte {

// ----------------------------------------------------------------------------

common::ranked_symbol < > GlushkovIndexate::getSymbolFromGlushkovPair ( const common::ranked_symbol < > & symbol ) {
	const ext::pair < DefaultSymbolType, DefaultSymbolType > & sps = ( ( const object::AnyObject < ext::pair < DefaultSymbolType, DefaultSymbolType > > & ) symbol.getSymbol ( ).getData ( ) ).getData ( );
	return common::ranked_symbol < > ( sps.first, symbol.getRank ( ) );
}

FormalRTE < > GlushkovIndexate::index ( const rte::FormalRTE < > & rte ) {
	int i = 1;

	return FormalRTE < > ( FormalRTEStructure < DefaultSymbolType, unsigned > ( rte.getRTE ( ).getStructure ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i ) ) );
}

// ----------------------------------------------------------------------------

ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESymbolAlphabet < DefaultSymbolType, unsigned > & node, int & i ) {
	ext::pair < DefaultSymbolType, DefaultSymbolType > sps =  ext::make_pair ( DefaultSymbolType ( node.getSymbol ( ).getSymbol ( ) ), DefaultSymbolType ( i++ ) );
	ext::ptr_vector < rte::FormalRTEElement < DefaultSymbolType, unsigned > > children;
	for ( const rte::FormalRTEElement < DefaultSymbolType, unsigned > & e : node.getElements ( ) ) {
		children.push_back ( ( FormalRTEElement < DefaultSymbolType, unsigned > && ) e.accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i ) );
	}

	return ext::ptr_value < FormalRTESymbolAlphabet < DefaultSymbolType, unsigned > > ( FormalRTESymbolAlphabet < DefaultSymbolType, unsigned > ( common::ranked_symbol < > ( DefaultSymbolType ( object::AnyObject < ext::pair < DefaultSymbolType, DefaultSymbolType > > ( sps ) ), node.getSymbol ( ).getRank ( ) ), std::move ( children ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESymbolSubst < DefaultSymbolType, unsigned > & node, int & ) {
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > ( FormalRTESymbolSubst < DefaultSymbolType, unsigned > ( node ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEAlternation < DefaultSymbolType, unsigned > & node, int & i ) {
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > > left = node.getLeftElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i );
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > > right = node.getRightElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > ( FormalRTEAlternation < DefaultSymbolType, unsigned > ( std::move ( left ), std::move ( right ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTESubstitution < DefaultSymbolType, unsigned > & node, int & i ) {
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > > left = node.getLeftElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i );
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > > right = node.getRightElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > ( FormalRTESubstitution < DefaultSymbolType, unsigned > ( std::move ( left ), std::move ( right ), node.getSubstitutionSymbol ( ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEIteration < DefaultSymbolType, unsigned > & node, int & i ) {
	ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > > element = node.getElement ( ).accept < ext::ptr_value < rte::FormalRTEElement < DefaultSymbolType, unsigned > >, GlushkovIndexate::Formal > ( i );
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > ( FormalRTEIteration < DefaultSymbolType, unsigned > ( std::move ( element ), node.getSubstitutionSymbol ( ) ) );
}

ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > GlushkovIndexate::Formal::visit ( const rte::FormalRTEEmpty < DefaultSymbolType, unsigned > &, int & ) {
	return ext::ptr_value < FormalRTEElement < DefaultSymbolType, unsigned > > ( FormalRTEEmpty < DefaultSymbolType, unsigned > ( ) );
}

// ----------------------------------------------------------------------------

} /* namespace rte */
