#include "RegExpDerivationTest.h"

#include "regexp/transform/RegExpDerivation.h"

#include <factory/StringDataFactory.hpp>
#include "factory/XmlDataFactory.hpp"

#include <regexp/string/UnboundedRegExp.h>
#include <regexp/xml/UnboundedRegExp.h>

#include <string/string/LinearString.h>
#include <string/xml/LinearString.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( RegExpDerivationTest, "regexp" );
CPPUNIT_TEST_SUITE_REGISTRATION( RegExpDerivationTest );

void RegExpDerivationTest::setUp() {
}

void RegExpDerivationTest::tearDown() {
}

void RegExpDerivationTest::testRegExpDerivation() {
    // https://edux.fit.cvut.cz/oppa/BI-AAG/cviceni/aag-cv04.pdf

    // Exc. 13
    this->ExecSingleTest("0 1 0 + 1 0 1 + 0*1 + 1*0", "0", "#E 1 0+#0 0 1+#E 0* 1+#0+#0 1* 0+#E");
    this->ExecSingleTest("0 1 0 + 1 0 1 + 0*1 + 1*0", "0 0", "#0 1 0+#0 0+#0 0 1+(#0 0*+#E 0*) 1+#0+#0+#0 1* 0+#0");

    // Exc. 14
    this->ExecSingleTest("(0 1*0 + 1 0*)*", "1", "(#0 1* 0+#E 0*) (0 1* 0+1 0*)*");
    this->ExecSingleTest("(0 1*0 + 1 0*)*", "1 0", "(#0 1* 0+#0 0*+#E 0*) (0 1* 0+1 0*)*+(#E 1* 0+#0 0*) (0 1* 0+1 0*)*");

    // Exc. 15
    this->ExecSingleTest("1 0*1*0", "1 0 0", "#0 0* 1* 0+(#0 0*+#E 0*) 1* 0+#0 1* 0+#E+#0 1* 0+#0");
}

void RegExpDerivationTest::ExecSingleTest(std::string regexp_str, std::string string_str, std::string result) {
    regexp::UnboundedRegExp < > regexp0 = factory::StringDataFactory::fromString (regexp_str);
    ext::deque<sax::Token> tokens = factory::XmlDataFactory::toTokens(regexp0);
    regexp::UnboundedRegExp < > regexp = factory::XmlDataFactory::fromTokens (std::move(tokens));

    string::LinearString < > string0 = factory::StringDataFactory::fromString ("\"" + string_str + "\"");
    ext::deque<sax::Token> tokens2 = factory::XmlDataFactory::toTokens(string0);
    string::LinearString < > string1 = factory::XmlDataFactory::fromTokens (std::move(tokens2));

    std::cout << factory::StringDataFactory::toString(regexp::RegExpDerivation::derivation(regexp, string1)) << " == " << std::endl << result << std::endl << std::endl;

    CPPUNIT_ASSERT(factory::StringDataFactory::toString(regexp::RegExpDerivation::derivation(regexp, string1)) == result);
}
