#include "GeneralizedLevenshteinDynamicProgrammingTest.h"

#include <string/LinearString.h>
#include <stringology/simulations/GeneralizedLevenshteinDynamicProgramming.h>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( GeneralizedLevenshteinDynamicProgrammingTest, "dynamic programming" );
CPPUNIT_TEST_SUITE_REGISTRATION ( GeneralizedLevenshteinDynamicProgrammingTest );

void GeneralizedLevenshteinDynamicProgrammingTest::testTableConstruction() {
  auto text = string::LinearString<>("adbcbaabadbbca");
  auto pattern = string::LinearString<>("adbbca");

  ext::vector<ext::vector<unsigned int>> expected_result = {
    ext::vector<unsigned int>({0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}),
    ext::vector<unsigned int>({1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0}),
    ext::vector<unsigned int>({2, 1, 0, 1, 2, 2, 1, 1, 1, 1, 0, 1, 2, 2, 1}),
    ext::vector<unsigned int>({3, 2, 1, 0, 1, 2, 2, 2, 1, 2, 1, 0, 1, 2, 2}),
    ext::vector<unsigned int>({4, 3, 2, 1, 1, 1, 2, 3, 2, 2, 2, 1, 0, 1, 2}),
    ext::vector<unsigned int>({5, 4, 3, 2, 1, 1, 2, 3, 3, 3, 3, 2, 1, 0, 1}),
    ext::vector<unsigned int>({6, 5, 4, 3, 2, 2, 1, 2, 4, 3, 4, 3, 2, 1, 0}),
  };

  CPPUNIT_ASSERT(expected_result == stringology::simulations::GeneralizedLevenshteinDynamicProgramming::compute_table(text, pattern));
}

void GeneralizedLevenshteinDynamicProgrammingTest::testSearch() {
  auto text = string::LinearString<>("adbcbaabadbbca");
  auto pattern = string::LinearString<>("adbbca");

  ext::set<unsigned int> expected_result = {2, 3, 4, 5, 6, 8, 10, 11, 12, 13};
  auto result = stringology::simulations::GeneralizedLevenshteinDynamicProgramming::search(text, pattern, 3);

  CPPUNIT_ASSERT(expected_result == result);
}

void GeneralizedLevenshteinDynamicProgrammingTest::setUp() { }

void GeneralizedLevenshteinDynamicProgrammingTest::tearDown() { }
