#include <lexer/Lexer.h>

#include <cctype>

namespace cli {

Lexer::Token Lexer::nextToken ( bool readNextLine ) {
	Token res { "", TokenType::ERROR };

q0:	if ( m_source->isEndOfTransmition ( ) ) {
		res.m_type = TokenType::EOT;
		return res;
	}
	if ( m_source->isEndOfSequence ( ) ) {
		res.m_type = TokenType::EOS;
		return res;
	}
	if ( isspace ( m_source->getCharacter ( ) ) ) {
		m_source->advance ( readNextLine );
		goto q0;
	}
	if ( m_source->getCharacter ( ) == '<' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::IN_REDIRECT;
		return res;
	}
	if ( m_source->getCharacter ( ) == '>' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::OUT_REDIRECT;
		return res;
	}
	if ( m_source->getCharacter ( ) == '(' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::LEFT_PAREN;
		return res;
	}
	if ( m_source->getCharacter ( ) == ')' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::RIGHT_PAREN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '{' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::LEFT_BRACE;
		return res;
	}
	if ( m_source->getCharacter ( ) == '}' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::RIGHT_BRACE;
		return res;
	}
	if ( m_source->getCharacter ( ) == '[' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::LEFT_BRACKET;
		return res;
	}
	if ( m_source->getCharacter ( ) == ']' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::RIGHT_BRACKET;
		return res;
	}
	if ( m_source->getCharacter ( ) == '@' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::AT_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '$' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::DOLAR_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '&' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::AMPERSAND_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '|' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::PIPE_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '^' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::CARET_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == ':' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::COLON_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '=' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::EQUAL_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '#' ) {
		m_source->advance ( readNextLine );
		res.m_type = TokenType::HASH_SIGN;
		return res;
	}
	if ( m_source->getCharacter ( ) == '-' ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q2;
	}

	if ( m_source->getCharacter ( ) == '"' ) {
		res.m_type = TokenType::STRING;
		m_source->advance ( true );
		goto q4;
	}

	if ( ( m_source->getCharacter ( ) >= '0' && m_source->getCharacter ( ) <= '9' ) ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q1;
	}

	if ( ( m_source->getCharacter ( ) >= 'a' && m_source->getCharacter ( ) <= 'z' )
	  || ( m_source->getCharacter ( ) >= 'A' && m_source->getCharacter ( ) <= 'Z' )
	  ||   m_source->getCharacter ( ) == '/' || m_source->getCharacter ( ) == '.'
	  ||   m_source->getCharacter ( ) == '~' || m_source->getCharacter ( ) == '_' ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q3;
	}

	res.m_type = TokenType::ERROR;
	return res;

q1:	if ( m_source->isEndOfSequence ( ) ) {
		res.m_type = TokenType::INTEGER;
		return res;
	}
	if ( ( m_source->getCharacter ( ) >= '0' && m_source->getCharacter ( ) <= '9' ) ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q1;
	} else {
		res.m_type = TokenType::INTEGER;
		return res;
	}

q2:	if ( m_source->isEndOfSequence ( ) ) {
		res.m_value = "";
		res.m_type = TokenType::DASH_SIGN;
		return res;
	}
	if ( ( m_source->getCharacter ( ) >= '0' && m_source->getCharacter ( ) <= '9' ) ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q1;
	}

	if ( ( m_source->getCharacter ( ) >= 'a' && m_source->getCharacter ( ) <= 'z' )
	  || ( m_source->getCharacter ( ) >= 'A' && m_source->getCharacter ( ) <= 'Z' )
	  ||   m_source->getCharacter ( ) == '/' || m_source->getCharacter ( ) == '.' || m_source->getCharacter ( ) == '-'
	  ||   m_source->getCharacter ( ) == '~' || m_source->getCharacter ( ) == '_' || m_source->getCharacter ( ) == ':' ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q3;
	}

	res.m_value = "";
	res.m_type = TokenType::DASH_SIGN;
	return res;

q3:	if ( m_source->isEndOfSequence ( ) ) {
		res.m_type = is_kw ( res.m_value );
		return res;
	}
	if ( ( m_source->getCharacter ( ) >= '0' && m_source->getCharacter ( ) <= '9' )
	  || ( m_source->getCharacter ( ) >= 'a' && m_source->getCharacter ( ) <= 'z' )
	  || ( m_source->getCharacter ( ) >= 'A' && m_source->getCharacter ( ) <= 'Z' )
	  ||   m_source->getCharacter ( ) == '/' || m_source->getCharacter ( ) == '.' || m_source->getCharacter ( ) == '-'
	  ||   m_source->getCharacter ( ) == '~' || m_source->getCharacter ( ) == '_' || m_source->getCharacter ( ) == ':' ) {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( readNextLine );
		goto q3;
	} else {
		res.m_type = is_kw ( res.m_value );
		return res;
	}

q4:	if ( m_source->isEndOfSequence ( ) ) {
		res.m_type = TokenType::ERROR;
		return res;
	}
	if ( m_source->getCharacter ( ) == '"' ) {
		m_source->advance ( readNextLine );
		goto q6;
	}
	if ( m_source->getCharacter ( ) == '\\' ) {
		m_source->advance ( true );
		goto q5;
	} else {
		res.m_value += m_source->getCharacter ( );
		m_source->advance ( true );
		goto q4;
	}

q5:	if ( m_source->isEndOfSequence ( ) ) {
		res.m_type = TokenType::ERROR;
		return res;
	}

	if ( m_source->getCharacter ( ) == 'n' ) {
		res.m_value += '\n';
	} else if ( m_source->getCharacter ( ) == 't' ) {
		res.m_value += '\t';
	} else if ( m_source->getCharacter ( ) == '"' ) {
		res.m_value += '"';
	} else {
		res.m_value += m_source->getCharacter ( );
	}

	m_source->advance ( true );
	goto q4;

q6:	if ( m_source->isEndOfSequence ( ) ) {
		return res;
	}
	if ( isspace ( m_source->getCharacter ( ) ) ) {
		m_source->advance ( readNextLine );
		goto q6;
	}
	if ( m_source->getCharacter ( ) == '"' ) {
		m_source->advance ( true );
		goto q4;
	} else {
		return res;
	}
}

} /* namespace cli */
