/*
 * CommonException.cpp
 *
 * Created on: Apr 1, 2013
 * Author: Martin Zak
 */

#include "CommonException.h"

#include <global/GlobalData.h>

#include <registration/ValuePrinterRegistration.hpp>

#ifdef DEBUG
	#include <debug/simpleStacktrace.h>
#endif

namespace exception {

CommonException::CommonException ( std::string cause ) : m_cause(std::move(cause)) {
	#ifdef DEBUG
		this->m_backtrace = ext::simpleStacktrace();
	#else
		this->m_backtrace = "";
	#endif

	for(int i = 0; i < common::GlobalData::argc; i++) {
		if(i != 0) this->m_command += " ";
		this->m_command += common::GlobalData::argv[i];
	}
	this->m_whatMessage += this->m_backtrace;
	this->m_whatMessage += this->m_cause;
}

CommonException::CommonException ( std::string cause, std::string backtrace, std::string command ) : m_cause(std::move(cause)), m_backtrace(std::move(backtrace)), m_command(std::move(command)) {
	this->m_whatMessage += this->m_backtrace;
	this->m_whatMessage += this->m_cause;
}

int CommonException::compare(const CommonException& other) const {
	return this->m_whatMessage.compare(other.m_whatMessage);
}

const char * CommonException::what ( ) const noexcept {
	return m_whatMessage.c_str ( );
}

const std::string & CommonException::getCause ( ) const {
	return m_cause;
}

const std::string & CommonException::getBacktrace ( ) const {
	return m_backtrace;
}

const std::string & CommonException::getCommand ( ) const {
	return m_command;
}

std::ostream & operator << ( std::ostream & os, const CommonException & data ) {
	return os << data.what ( );
}

CommonException::operator std::string ( ) const {
	return this->m_whatMessage;
}

} /* namespace exception */

namespace {

static auto valuePrinter = registration::ValuePrinterRegister < exception::CommonException > ( );

} /* namespace */
