/*
 * PrimitiveRegistrator.cpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <registry/ContainerRegistry.hpp>

#include <registration/XmlRegistration.hpp>
#include <registry/XmlContainerParserRegistry.hpp>

#include <primitive/xml/UnsignedLong.h>

#include <container/xml/ObjectsSet.h>
#include <container/xml/ObjectsMap.h>
#include <container/xml/ObjectsVector.h>
#include <container/xml/ObjectsVariant.h>

#include <common/ranked_symbol.hpp>
#include <alphabet/xml/RankedSymbol.h>

#include <string/xml/LinearString.h>
#include <string/xml/Epsilon.h>

namespace {

class PrimitiveRegistrator {
public:
	PrimitiveRegistrator ( ) {
		abstraction::ContainerRegistry::registerSet < common::ranked_symbol < object::Object, unsigned > > ( "RankedSymbol" );
		abstraction::XmlContainerParserRegistry::registerSet < common::ranked_symbol < object::Object, unsigned > > ( "RankedSymbol" );

		registration::XmlWriterRegister < ext::set < common::ranked_symbol < object::Object, unsigned > > > ( );
		registration::XmlWriterRegister < common::ranked_symbol < object::Object, unsigned > > ( );
		registration::XmlWriterRegister < ext::vector < ext::map < std::pair < object::Object, object::Object >, ext::map < object::Object, object::Object > > > > ( );
		registration::XmlWriterRegister < ext::vector < ext::vector < ext::set < object::Object > > > > ( );
		registration::XmlWriterRegister < ext::map < common::ranked_symbol < object::Object, unsigned >, size_t > > ( );
		registration::XmlWriterRegister < ext::set < string::LinearString < > > > ( );

		abstraction::XmlParserRegistry::registerXmlParser < object::Object > ( "DefaultStateType" );

		core::xmlApi < object::Object >::template registerXmlWriter < ext::set < common::ranked_symbol < object::Object, unsigned > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::pair < ext::set < ext::pair < object::Object, object::Object > >, ext::variant < string::Epsilon < object::Object >, object::Object > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::pair < object::Object, ext::variant < string::Epsilon < object::Object >, object::Object > > > ( );
	}
};

auto primitiveRegistrator = PrimitiveRegistrator ( );

} /* namespace */
