#include "RegistryTest.h"
#include <Algorithm/Registry.hpp>

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( RegistryTest, "registry" );
CPPUNIT_TEST_SUITE_REGISTRATION ( RegistryTest );

void RegistryTest::setUp ( ) {
}

void RegistryTest::tearDown ( ) {
}

void RegistryTest::registryTest ( ) {
	Registry::deinitialize();

	CPPUNIT_ASSERT(Registry::getAlgorithm("automaton::determinize::Determinize") == nullptr);

	Registry::initialize();

	auto* algo = Registry::getAlgorithm("automaton::determinize::Determinize");
	CPPUNIT_ASSERT(algo != nullptr);

	CPPUNIT_ASSERT(Registry::getAlgorithm("nonexistent_algorithm") == nullptr);
}

void RegistryTest::algorithmTestBasic ( ) {
	auto name = "automaton::determinize::Determinize";

	auto* algo = Registry::getAlgorithm(name);

	CPPUNIT_ASSERT(algo->getFullName() == name);
	CPPUNIT_ASSERT(algo->getPrettyName() == "Determinize");

	const auto& groups = algo->getGroups();
	CPPUNIT_ASSERT(groups[0] == "automaton");
	CPPUNIT_ASSERT(groups[1] == "determinize");

	CPPUNIT_ASSERT(algo->getInputCount() == 1);
}

void RegistryTest::algorithmTestInputCount ( ) {
	auto* algo = Registry::getAlgorithm("automaton::transform::AutomataConcatenation");

	CPPUNIT_ASSERT(algo != nullptr);
	CPPUNIT_ASSERT(algo->getInputCount() == 2);
}

void RegistryTest::algorithmTestPrettyName ( ) {
	auto* algo = Registry::getAlgorithm("automaton::transform::AutomataConcatenationEpsilonTransition");

	CPPUNIT_ASSERT(algo != nullptr);
	CPPUNIT_ASSERT(algo->getPrettyName() == u8"\u03B5-Concatenate");
}
