/*
 * PrimitiveRegistrator.cpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#include <registration/XmlRegistration.hpp>
#include <registry/XmlContainerParserRegistry.hpp>
#include <registry/ContainerRegistry.hpp>

#include <primitive/xml/Character.h>
#include <primitive/xml/Double.h>
#include <primitive/xml/Integer.h>
#include <primitive/xml/Unsigned.h>
#include <primitive/xml/Bool.h>
#include <primitive/xml/UnsignedLong.h>
#include <primitive/xml/String.h>

#include <container/xml/ObjectsVector.h>
#include <container/xml/ObjectsSet.h>
#include <container/xml/ObjectsVariant.h>
#include <container/xml/ObjectsPair.h>
#include <container/xml/ObjectsMap.h>

namespace {

class PrimitiveRegistrator {
public:
	PrimitiveRegistrator ( ) {
		abstraction::XmlContainerParserRegistry::registerSet < int > ( );
		abstraction::XmlParserRegistry::registerXmlParser < int > ( "int" );
		abstraction::XmlParserRegistry::registerXmlParser < ext::set < ext::pair < object::Object, object::Object > > > ( "pair_set" );

		registration::XmlWriterRegister < ext::vector < size_t > > ( );
		registration::XmlWriterRegister < ext::vector < unsigned > > ( );
		registration::XmlWriterRegister < ext::vector < int > > ( );
		registration::XmlWriterRegister < ext::set < size_t > > ( );
		registration::XmlWriterRegister < ext::set < unsigned > > ( );
		registration::XmlWriterRegister < ext::set < int > > ( );
		registration::XmlWriterRegister < ext::map < object::Object, size_t > > ( );
		registration::XmlWriterRegister < object::Object > ( );
		registration::XmlWriterRegister < ext::set < object::Object > > ( );

		abstraction::ContainerRegistry::registerSet < object::Object > ( "Object" );
		abstraction::XmlContainerParserRegistry::registerSet < object::Object > ( "Object" );
		abstraction::XmlParserRegistry::registerXmlParser < object::Object > ( "Object" );

		core::xmlApi < object::Object >::template registerXmlWriter < ext::set < ext::pair < object::Object, object::Object > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::pair < ext::set < ext::pair < object::Object, object::Object > >, object::Object > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::vector < ext::variant < object::Object, object::Object > > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::pair < unsigned int, unsigned int > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::pair < object::Object, unsigned int > > ( );
		core::xmlApi < object::Object >::template registerXmlWriter < ext::set < char > > ( );
	}
};

auto primitiveRegistrator = PrimitiveRegistrator ( );

} /* namespace */
