#include "ContainerTest.h"

#include <container/xml/ObjectsSet.h>
#include <container/xml/ObjectsVariant.h>
#include <container/xml/ObjectsTree.h>
#include <container/xml/ObjectsTrie.h>
#include <primitive/xml/Integer.h>
#include <primitive/xml/Character.h>
#include <primitive/xml/String.h>
#include <object/Object.h>

#include "factory/XmlDataFactory.hpp"

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION ( ContainerTest, "container" );
CPPUNIT_TEST_SUITE_REGISTRATION ( ContainerTest );

void ContainerTest::setUp ( ) {
}

void ContainerTest::tearDown ( ) {
}

void ContainerTest::testXMLParser ( ) {

	object::Object tmp ( object::AnyObject < std::string > ( "1" ) );

	ext::set < object::Object > set;

	set.insert ( tmp );

	object::Object object { object::AnyObject < ext::set < object::Object > > ( set ) };

	{
		std::string tmp2 = factory::XmlDataFactory::toString ( object );

		std::cout << tmp2 << std::endl;

		object::Object object2 = factory::XmlDataFactory::fromString ( tmp2 );

		CPPUNIT_ASSERT ( object == object2 );

		ext::set < std::string > concrete = factory::XmlDataFactory::fromString ( tmp2 );
		std::string tmp3 = factory::XmlDataFactory::toString ( concrete );

		std::cout << tmp3 << std::endl;

		object::Object object3 = factory::XmlDataFactory::fromString ( tmp3 );

		CPPUNIT_ASSERT ( object == object3 );
	}
}

void ContainerTest::testVariantParsing ( ) {
	{
		std::string string = "<String>aaa</String>";
		ext::variant < int, std::string > object = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( "aaa" == object.get < std::string > ( ) );

		std::string string2 = factory::XmlDataFactory::toString ( object );

		ext::variant < int, std::string > object2 = factory::XmlDataFactory::fromString ( string2 );

		CPPUNIT_ASSERT ( object == object2 );
	}
}

void ContainerTest::testTreeParsing ( ) {
	{
		ext::tree < int > t ( 1 );

		std::string string = factory::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		ext::tree < int > t2 = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		ext::tree < int > t ( 1, ext::tree < int > ( 2 ), ext::tree < int > ( 4 ) );

		std::string string = factory::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		ext::tree < int > t2 = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		ext::tree < int > t ( 1, ext::tree < int > ( 2, ext::tree < int > ( 3 ) ), ext::tree < int > ( 4 ) );

		std::string string = factory::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		ext::tree < int > t2 = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
}

void ContainerTest::testTrieParsing ( ) {
	{
		ext::trie < char, int > t ( 1 );

		std::string string = factory::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		ext::trie < char, int > t2 = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		ext::trie < char, int > t ( 0, ext::map < char, ext::trie < char, int > > { std::make_pair ( 'a', ext::trie < char, int > ( 1 ) ), std::make_pair ( 'b', ext::trie < char, int > ( 2 ) ) } );

		std::string string = factory::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		ext::trie < char, int > t2 = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
	{
		ext::trie < char, int > t ( 0, ext::map < char, ext::trie < char, int > > { std::make_pair ( 'a', ext::trie < char, int > ( 1, ext::map < char, ext::trie < char, int > > { std::make_pair ( 'a', ext::trie < char, int > ( 3 ) ), std::make_pair ( 'b', ext::trie < char, int > ( 4 ) ) } ) ), std::make_pair ( 'b', ext::trie < char, int > ( 2, ext::map < char, ext::trie < char, int > > { std::make_pair ( 'a', ext::trie < char, int > ( 5 ) ), std::make_pair ( 'b', ext::trie < char, int > ( 6 ) ) } ) ) } );

		std::string string = factory::XmlDataFactory::toString ( t );
		std::cout << string << std::endl;

		ext::trie < char, int > t2 = factory::XmlDataFactory::fromString ( string );

		CPPUNIT_ASSERT ( t == t2 );
	}
}
