/*
 * Prompt.h
 *
 *  Created on: 20. 3. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _AQL_PROMPT_H_
#define _AQL_PROMPT_H_

#include <string>

#include <lexer/Lexer.h>
#include <parser/Parser.h>

#include <factory/XmlDataFactory.hpp>
#include <global/GlobalData.h>

#include "HistoryRegister.h"
#include "ReadlineCharSequence.h"

class Prompt {
	Prompt ( const Prompt & ) = delete;
	Prompt ( Prompt && ) = delete;
	Prompt & operator = ( const Prompt & ) = delete;
	Prompt & operator = ( Prompt && ) = delete;

	std::string m_history_file;

	cli::Environment m_environment;

public:
	Prompt ( cli::Environment environment );
	~Prompt ( );

	cli::Command::Result run ( );

	template < class T >
	cli::Command::Result execute_line ( T && charSequence, bool allowHistory ) {
		try {
			cli::Parser parser = cli::Parser ( cli::Lexer ( std::forward < T && > ( charSequence ) ) );
			HistoryRegister historyRegister ( parser.getLexer ( ), allowHistory );
			return parser.parse ( )->run ( m_environment );
		} catch ( const exception::CommonException & exception ) {
			factory::XmlDataFactory::toStdout ( exception );
			return cli::Command::Result::EXCEPTION;
		} catch ( const std::exception & exception ) {
			common::Streams::err << "Exception caught: " << exception.what ( ) << std::endl;
			return cli::Command::Result::EXCEPTION;
		}
	}

	const cli::Environment & getEnvironment ( ) const {
		return m_environment;
	}
};

#endif /* _AQL_PROMPT_H_ */
