/*
 * ReadlineCharSequence.h
 *
 *  Created on: 20. 3. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _READLINE_CHAR_SEQUENCE_H_
#define _READLINE_CHAR_SEQUENCE_H_

#include <cctype>
#include <string>
#include <vector>
#include <algorithm>

#include <readline/readline.h>

#include <lexer/CharSequenceBase.h>

class ReadlineCharSequence : public cli::CharSequenceBase {
	std::vector < std::string > m_lines;
	bool first = true;
	bool forceReadLine = false;

	virtual void fetch ( bool readNextLine ) override {
		if ( readNextLine == false && forceReadLine == false ) {
			this->endOfSequence = true;
			return;
		}

		char * read = ::readline ( first ? "> ": "+ " );
		if ( ! read ) {
			this->endOfTransmition = true;
			return;
		}

		std::string line ( read );
		free ( read );

		forceReadLine = line.size ( ) > 0 && line.at ( line.size ( ) - 1 ) == '\\';
		if ( forceReadLine )
			line.back ( ) = '\n';
		else
			line += '\n';

		if ( ! std::all_of ( line.begin ( ), line.end ( ), isspace ) )
			first = false;

		m_lines.push_back ( std::move ( line ) );

		this->linePtr = m_lines.back ( ).c_str ( );
	}

public:
	ReadlineCharSequence ( ) {
		fetch ( true );
	}

	ReadlineCharSequence ( ReadlineCharSequence && ) = default;

	ReadlineCharSequence ( const ReadlineCharSequence & ) = delete;

	ReadlineCharSequence & operator = ( ReadlineCharSequence && ) = delete;

	ReadlineCharSequence & operator = ( const ReadlineCharSequence & ) = delete;

	virtual std::string getLine ( ) const override {
		std::string res;
		for ( const std::string & line : m_lines ) {
			res += line;
		}
		return res;
	}
};

#endif /* _READLINE_CHAR_SEQUENCE_H_ */
