#
# spec file for package algorithms-library
#
# This file is part of Algorithms library toolkit.
# Copyright (C) 2018 Jan Travnicek (jan.travnicek@fit.cvut.cz)
#
# Algorithms library toolkit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Algorithms library toolkit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Algorithms library toolkit.  If not, see <http://www.gnu.org/licenses/>.
#

# adresář ve staženém tar.gz z _service se zdrojáky obsahuje adresář který musí mít jméno ${Name}-${Version}

Name:           algorithms-library
Version:        0.0.1
Release:        2%{?dist}
Summary:        Theoretical computer science datastructures and algorithms implementation
License:        GPL-3.0-or-later
Group:          Productivity/Scientific/Math
Url:            https://gitlab.fit.cvut.cz/algorithms-library-toolkit/automata-library
Source0:        algorithms-library.tar.gz
Patch0:         remove-werror.patch
BuildRequires:  python3 cmake gcc-c++ curl bash make libxml2-devel cppunit-devel tclap readline-devel libQt5Widgets-devel libQt5Xml-devel graphviz-devel jsoncpp-devel
Requires:       readline

%description
Theoretical computer science datastructures and algorithms implementation.

The library provides graphical and console interface similar to bash with many classical algorithms manipulating automata, grammars, regular expressions, trees and strings.

#------------------------------------------------------------------

%package devel
Summary:        Development files for algorithms-library
Group:          Development/Libraries/C and C++
Requires:       algorithms-library = 0.0.1

%description devel
Theoretical computer science datastructures and algorithms implementation development sources.

The library provides graphical and console interface similar to bash with many classical algorithms manipulating automata, grammars, regular expressions, trees and strings.

%prep
%setup -q
%patch0 -p0

%build
./CMake/generate.py -w -m

mkdir debug
cd debug
cmake -DCMAKE_BUILD_TYPE=Release -DALIB_PRODUCTION=1 ..
make -j6

%install
cd debug
%makeinstall

cd ..
for library in alib2std alib2abstraction alib2algo alib2algo_experimental alib2aux alib2cli alib2common alib2data alib2data_experimental  alib2dummy alib2elgo alib2graph_algo alib2graph_data alib2gui alib2measure alib2raw alib2raw_cli_integration alib2std alib2str alib2str_cli_integration alib2xml; do
  find ${library}/src/ -name "*\.hpp" -o -name "*\.h" -o -name "*\.hxx" | while read line; do
    if [ -s "$line" ]; then
      target=$(dirname ${RPM_BUILD_ROOT}/usr/include/algorithms-library/`echo $line | cut -sd / -f 3-`)
      mkdir -p $target && cp $line $target
    fi
  done
done
cd alib2std/src/
cp -r alib ${RPM_BUILD_ROOT}/usr/include/algorithms-library/

%post
/sbin/ldconfig

%postun
/sbin/ldconfig

%files
%defattr(-,root,root)
/usr/bin/aaccess2
/usr/bin/aarbology2
/usr/bin/acast2
/usr/bin/acompaction2
/usr/bin/acompare2
/usr/bin/aconversions2
/usr/bin/aconvert2
/usr/bin/aderivation2
/usr/bin/adeterminize2
/usr/bin/aecho2
/usr/bin/aepsilon2
/usr/bin/agenerate2
/usr/bin/agui2
/usr/bin/aintegral2
/usr/bin/aintrospection2
/usr/bin/alangop2
/usr/bin/aminimize2
/usr/bin/anormalize2
/usr/bin/aql2
/usr/bin/aquery2
/usr/bin/arand2
/usr/bin/araw2
/usr/bin/arename2
/usr/bin/areverse2
/usr/bin/arun2
/usr/bin/astat2
/usr/bin/astringology2
/usr/bin/atrim2
/usr/bin/tniceprint

/usr/lib/libalib2abstraction.so.0
/usr/lib/libalib2abstraction.so.0.0.1

/usr/lib/libalib2algo.so.0
/usr/lib/libalib2algo.so.0.0.1

/usr/lib/libalib2algo_experimental.so.0
/usr/lib/libalib2algo_experimental.so.0.0.1

/usr/lib/libalib2aux.so.0
/usr/lib/libalib2aux.so.0.0.1

/usr/lib/libalib2cli.so.0
/usr/lib/libalib2cli.so.0.0.1

/usr/lib/libalib2common.so.0
/usr/lib/libalib2common.so.0.0.1

/usr/lib/libalib2data.so.0
/usr/lib/libalib2data.so.0.0.1

/usr/lib/libalib2data_experimental.so.0
/usr/lib/libalib2data_experimental.so.0.0.1

/usr/lib/libalib2dummy.so.0
/usr/lib/libalib2dummy.so.0.0.1

/usr/lib/libalib2elgo.so.0
/usr/lib/libalib2elgo.so.0.0.1

/usr/lib/libalib2graph_algo.so.0
/usr/lib/libalib2graph_algo.so.0.0.1

/usr/lib/libalib2graph_data.so.0
/usr/lib/libalib2graph_data.so.0.0.1

/usr/lib/libalib2gui.so.0
/usr/lib/libalib2gui.so.0.0.1

/usr/lib/libalib2measure.so.0
/usr/lib/libalib2measure.so.0.0.1

/usr/lib/libalib2raw.so.0
/usr/lib/libalib2raw.so.0.0.1

/usr/lib/libalib2raw_cli_integration.so.0
/usr/lib/libalib2raw_cli_integration.so.0.0.1

/usr/lib/libalib2std.so.0
/usr/lib/libalib2std.so.0.0.1

/usr/lib/libalib2str.so.0
/usr/lib/libalib2str.so.0.0.1

/usr/lib/libalib2str_cli_integration.so.0
/usr/lib/libalib2str_cli_integration.so.0.0.1

/usr/lib/libalib2xml.so.0
/usr/lib/libalib2xml.so.0.0.1

%files devel
%defattr(-,root,root)
/usr/lib/libalib2abstraction.so
/usr/lib/libalib2algo.so
/usr/lib/libalib2algo_experimental.so
/usr/lib/libalib2aux.so
/usr/lib/libalib2cli.so
/usr/lib/libalib2common.so
/usr/lib/libalib2data.so
/usr/lib/libalib2data_experimental.so
/usr/lib/libalib2dummy.so
/usr/lib/libalib2elgo.so
/usr/lib/libalib2graph_algo.so
/usr/lib/libalib2graph_data.so
/usr/lib/libalib2gui.so
/usr/lib/libalib2measure.so
/usr/lib/libalib2raw.so
/usr/lib/libalib2raw_cli_integration.so
/usr/lib/libalib2std.so
/usr/lib/libalib2str.so
/usr/lib/libalib2str_cli_integration.so
/usr/lib/libalib2xml.so

%dir /usr/include/algorithms-library
/usr/include/algorithms-library/*

%changelog
* Wed Nov 15 2018 Jan Travnicek <Jan.Travnicek@fit.cvut.cz> 0.0.1-2
- Add devel package

* Wed Nov 14 2018 Jan Travnicek <Jan.Travnicek@fit.cvut.cz> 0.0.1-1
- Initial RPM release
